# -*- coding: utf-8 -*-
require "delayer/deferred/promise"
require "delayer/deferred/chain"
require "delayer/deferred/deferredable"
require "delayer/deferred/worker"
require "delayer/deferred/version"

module Delayer::Deferred
  Deferred = Promise
end
